@@ -3,7 +3,7 @@
 # this is a terminal-based game, run the `pperil` command that should be
 # installed with this module to start the game
 #
-# some details for the unwary, or brave, regarding the following code:
+# some details for the unwary, or brave, regarding the code:
 #
 # this implementation uses arrays heavily so instead of a more typical
 # Player object there is an array with various slots that are used for
@@ -22,13 +22,13 @@
 # level maps are ASCII text, and only one thing can be present in a cell
 # in the map (with FLOOR being assumed present below any item or
 # animate). there are some complications around killing things off; dead
-# things must not interact with anything, but may be looped to after
-# their dead in the apply_gravity or game_loop UPDATE calls. hence the
-# BLACK_SPOT                                                        
+# things must not interact with anything, but may still be looped to
+# after their death in the apply_gravity or game_loop UPDATE calls.
+# hence the BLACK_SPOT
 
 package Game::PlatformsOfPeril;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use 5.24.0;
 use warnings;
@@ -72,6 +72,7 @@ sub ANI ()    { 3 }
 
 sub MOVE_FAILED () { 0 }
 sub MOVE_OK ()     { 1 }
+sub MOVE_NEWLVL () { 2 }
 
 # for the level map
 sub COLS ()               { 23 }
@@ -82,8 +83,9 @@ sub MAP_DISPLAY_OFFSET () { 1 }
 sub PROW () { 1 }
 sub PCOL () { 0 }
 
-sub MESSAGE_ROW ()      { 1 }
-sub MESSAGE_COL ()      { 25 }
+sub MESSAGE_ROW () { 1 }
+sub MESSAGE_COL () { 25 }
+# these also used to determine the minimum size for the terminal
 sub MESSAGE_MAX ()      { 24 }
 sub MESSAGE_COLS_MAX () { 70 }
 
@@ -124,7 +126,10 @@ our %CharMap = (
     '#' => WALL,
 );
 
-our ( @Graphs, $LMap, $Monst_Name, @RedrawA, @RedrawB, $Hero, $TCols, $TRows );
+our (
+    @Death_Row, @Graphs, $LMap,  $Monst_Name, @RedrawA,
+    @RedrawB,   $Hero,   $TCols, $TRows
+);
 
 our $Level = 0;
 our $Level_Path;
@@ -226,8 +231,6 @@ our %Key_Commands = (
         rotate_left();
         print draw_level();
         sleep($Rotate_Delay);
-        # NOTE rotations confuse monsters (until the player falls, or
-        # makes a non-rotation move). did not like track_player() here
         return MOVE_OK;
     },
     '>' => sub {
@@ -241,6 +244,9 @@ our %Key_Commands = (
         post_help();
         return MOVE_FAILED;
     },
+    # for debugging, probably shouldn't be included as it shows exactly
+    # where the monsters are trying to move to which may or may not be
+    # where the player is
     'T' => sub {
         local $" = ',';
         post_message("T $Hero->@* R $Rotation");
@@ -259,12 +265,17 @@ our %Key_Commands = (
               . ' gems.' );
         return MOVE_FAILED;
     },
+    # by way of history '%' is what rogue (version 3.6) uses for stairs,
+    # except the '>' (or very rarely '<') keys are used to interact with
+    # that symbol
     '%' => sub {
         if ( $Animates{ HERO, }[LMC][GROUND][TYPE] == STAIR ) {
             load_level();
-            return MOVE_OK;
+            print clear_screen(), draw_level();
+            post_message( 'Level ' . $Level );
+            return MOVE_NEWLVL;
         } else {
-            post_message('There are no stairs here.');
+            post_message('There are no stairs here?');
             return MOVE_FAILED;
         }
     },
@@ -302,8 +313,8 @@ our %Key_Commands = (
 );
 
 sub apply_gravity {
-    my $hero_fall = 0;
     for my $ent ( rev_nsort_by { $_->[LMC][WHERE][PROW] } values %Animates ) {
+        next if $ent->[BLACK_SPOT];
         my $here = $ent->[LMC][WHERE];
         next
           if $here->[PROW] == ROWS - 1
@@ -318,15 +329,17 @@ sub apply_gravity {
             } else {
                 post_message('You fall!');
             }
-            $hero_fall = 1;
         }
     }
-    track_hero() if $hero_fall;
 }
 
 sub bad_terminal {
     ( $TCols, $TRows ) = (GetTerminalSize)[ 0, 1 ];
-    return ( not defined $TCols or $TCols < MESSAGE_COLS_MAX or $TRows < ROWS );
+    return (
+             not defined $TCols
+          or $TCols < MESSAGE_COLS_MAX
+          or $TRows < MESSAGE_MAX
+    );
 }
 
 sub bail_out {
@@ -356,15 +369,13 @@ sub draw_level {
 sub explode {
     post_message('ka-boom!');
     for my $ent (@_) {
-        if ( $ent->[LMC][GROUND][TYPE] == STATUE ) {
-            $ent->[LMC][GROUND] = $Things{ FLOOR, };
-        }
-        push @RedrawA, $ent->[LMC][WHERE];
-        $ent->[BLACK_SPOT] = 1;
-        undef $ent->[LMC][ $ent->[TYPE] ];
+        # HEROIC DESTRUCTION
+        $ent->[LMC][GROUND] = $Things{ FLOOR, } if $ent->[LMC][GROUND][TYPE] == STATUE;
+        kill_animate($ent);
     }
 }
 
+# cribbed from some A* article on https://www.redblobgames.com/
 sub find_hero {
     my ( $ent, $mcol, $mrow ) = @_;
 
@@ -414,7 +425,7 @@ sub find_hero {
 }
 
 sub game_loop {
-    game_over( 'Terminal must be at least ' . MESSAGE_COLS_MAX . 'x' . ROWS )
+    game_over( 'Terminal must be at least ' . MESSAGE_COLS_MAX . 'x' . MESSAGE_MAX )
       if bad_terminal();
     ( $Level_Path, $Level, $Seed ) = @_;
     $SIG{$_} = \&bail_out for qw(INT HUP TERM PIPE QUIT USR1 USR2 __DIE__);
@@ -424,6 +435,8 @@ sub game_loop {
     print term_norm, hide_cursor, hide_pointer, clear_screen, draw_level;
     post_message('The Platforms of Peril');
     post_message('');
+    post_message( 'Your constant foes, the ' . $Monst_Name . 's' );
+    post_message('seek to destroy your way of life!');
     post_help();
     post_message('');
     post_message( 'Seed ' . $Seed );
@@ -434,17 +447,16 @@ sub game_loop {
     };
 
     while (1) {
-        my @Death_Row;
         apply_gravity();
+        while ( my $id = pop @Death_Row ) { delete $Animates{$id} }
         redraw_movers() if @RedrawA;
-        for my $ent ( nsort_by { $_->[ANI_ID] } values %Animates ) {
-            if ( $ent->[BLACK_SPOT] ) {
-                push @Death_Row, $ent;
-                next;
-            }
-            $ent->[UPDATE]->($ent) if defined $ent->[UPDATE];
+        my @actors = nsort_by { $_->[ANI_ID] } values %Animates;
+        next if shift(@actors)->[UPDATE]->() == MOVE_NEWLVL;
+        track_hero();
+        for my $ent (@actors) {
+            $ent->[UPDATE]->($ent) if !$ent->[BLACK_SPOT] and defined $ent->[UPDATE];
         }
-        for my $ent (@Death_Row) { delete $Animates{ $ent->[ANI_ID] } }
+        while ( my $id = pop @Death_Row ) { delete $Animates{$id} }
         redraw_movers();
     }
 }
@@ -459,9 +471,7 @@ sub game_over {
     exit $code;
 }
 
-sub game_over_bomb {
-    game_over('You gone done blowed yourself up.');
-}
+sub game_over_bomb { game_over('You gone done blowed yourself up.') }
 
 sub game_over_monster {
     game_over( 'The ' . $Monst_Name . ' polished you off.' );
@@ -470,9 +480,7 @@ sub game_over_monster {
 sub grab_gem {
     my ( $ent, $gem ) = @_;
     $ent->[STASH][GEM_STASH] += $gem->[STASH];
-    push @RedrawA, $gem->[LMC][WHERE];
-    $gem->[BLACK_SPOT] = 1;
-    undef $gem->[LMC][ITEM];
+    kill_animate($gem);
     if ( $ent->[WHAT] == MONST ) {
         post_message( 'The ' . $Monst_Name . ' grabs a gem.' );
     } else {
@@ -595,6 +603,17 @@ sub interact {
     return 0;
 }
 
+sub kill_animate {
+    my ($ent) = @_;
+    push @RedrawA, $ent->[LMC][WHERE];
+    $ent->[BLACK_SPOT] = 1;
+    push @Death_Row, $ent->[ANI_ID];
+    # NOTE this only works for TYPE of ANI or ITEM, may need to rethink
+    # how STATUE and STAIRS are handled if there are GROUND checks on
+    # TYPE as those abuse the TYPE field for other things (see %Things)
+    undef $ent->[LMC][ $ent->[TYPE] ];
+}
+
 sub load_level {
     my $file = File::Spec->catfile( $Level_Path, 'level' . $Level++ );
     game_over( 'No more levels.', 0 ) unless -e $file;
@@ -701,14 +720,11 @@ sub move_player {
     return sub {
         my ( $status, $msg ) = move_animate( $Animates{ HERO, }, $cols, $rows );
         post_message($msg) if $msg;
-        track_hero();
         return $status;
     };
 }
 
 sub post_help {
-    post_message( 'Your constant foes, the ' . $Monst_Name . 's' );
-    post_message('seek to destroy your way of life!');
     post_message('');
     post_message(
         ' ' . $Animates{ HERO, }[DISP] . ' - You   P - a ' . $Monst_Name );
@@ -847,8 +863,7 @@ sub track_hero {
 }
 
 sub update_hero {
-    my $key;
-    my %seen;
+    my ( $key, $ret, %seen );
     tcflush( STDIN_FILENO, TCIFLUSH );
     while (1) {
         while (1) {
@@ -857,10 +872,11 @@ sub update_hero {
             last if exists $Key_Commands{$key};
             post_message( sprintf "Illegal command \\%03o", ord $key );
         }
-        my ( $status, $msg ) = $Key_Commands{$key}->();
+        ( $ret, my $msg ) = $Key_Commands{$key}->();
         post_message($msg) if defined $msg;
-        last if $status == MOVE_OK;
+        last if $ret == MOVE_OK or $ret == MOVE_NEWLVL;
     }
+    return $ret;
 }
 
 sub update_monst {
@@ -868,7 +884,13 @@ sub update_monst {
     my $mcol  = $ent->[LMC][WHERE][PCOL];
     my $mrow  = $ent->[LMC][WHERE][PROW];
 
-    # prevent monster downward move where only gravity should apply
+    # prevent monster move where only gravity should apply
+    # NOTE one may have the clever idea that monsters can run across the
+    # heads of other monsters though that would require changes to how
+    # the graph is setup to permit such moves, and additional checks to
+    # see if something to tread upon is available (and then to let the
+    # hero do that (like in Lode Runner) or to prevent them from such
+    # head-running...)
     if (    $mrow != ROWS - 1
         and $ent->[LMC][GROUND][WHAT] == FLOOR
         and $LMap->[ $mrow + 1 ][$mcol][GROUND][WHAT] != WALL ) {
@@ -909,6 +931,9 @@ black background) install and run the game via:
     cpanm Game::PlatformsOfPeril
     pperil
 
+Help text should be printed when the game starts. Use the C<?> key in
+game to show the help text again.
+
 =head1 DESCRIPTION
 
 You are the only spawn (son, daughter, etc.) of a Xorbian Ranger and as
@@ -992,9 +1017,10 @@ C<$LMap> (which needs to know things about the animates). In other
 words, this game may leak memory.
 
 Some may be desirous of non-hjkl movement keys and so forth. Fiddle with
-the code, or abstract things to use MOVE_LEFT etc and then map keys to
-those symbols. At that point one might add a configuration file or
-in-game editor of the key commands, but that sounds like work.
+the code (C<our> variables can be clobbered from outside the module), or
+abstract things to use MOVE_LEFT etc and then map keys to those symbols.
+At that point one might add a configuration file or in-game editor of
+the key commands, but that sounds like work.
 
 Automatic level generation might be nice? Or more levels made by hand...
 
@@ -1010,6 +1036,21 @@ One idea is that a gem plus a bomb could make a smartbomb which, being
 smart, tracks the player. However bombs lack limbs so have trouble with
 the ladders, and that idea is otherwise presently tied up in committee.
 
+L<Game::TextPatterns> may help draw candidate level maps:
+
+    use Game::TextPatterns;
+
+    my $pat = Game::TextPatterns->new( pattern => <<'EOP' );
+    .==P..
+    o#===.
+    ####=.
+    =====.
+    .=*#..
+    .=###.
+    EOP
+
+    print $pat->four_up->flip_four(1)->string;
+
 =head1 AUTHOR
 
 thrig - Jeremy Mates (cpan:JMATES) C<< <jmates at cpan.org> >>
